/****************************************************************************************************************************************************
 * Copyright (c) 2015 Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 *    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************************************************************************************/

#include "SmallFont.hpp"
#include <cassert>

namespace Fsl
{
  namespace
  {
    FontGlyphRange g_ranges[] =
    {
      FontGlyphRange(32, 96),
    };


    FontGlyphBasicKerning g_glyphKerning[] =
    {
      FontGlyphBasicKerning(0, 0, 7),
      FontGlyphBasicKerning(1, 1, 9),
      FontGlyphBasicKerning(0, 1, 7),
      FontGlyphBasicKerning(1, 2, 12),
      FontGlyphBasicKerning(-1, -1, 12),
      FontGlyphBasicKerning(1, 1, 14),
      FontGlyphBasicKerning(0, 1, 13),
      FontGlyphBasicKerning(1, 1, 5),
      FontGlyphBasicKerning(1, 1, 7),
      FontGlyphBasicKerning(0, 1, 7),
      FontGlyphBasicKerning(2, 1, 12),
      FontGlyphBasicKerning(3, 6, 14),
      FontGlyphBasicKerning(1, 14, 7),
      FontGlyphBasicKerning(0, 10, 7),
      FontGlyphBasicKerning(1, 13, 7),
      FontGlyphBasicKerning(0, 0, 10),
      FontGlyphBasicKerning(1, 3, 12),
      FontGlyphBasicKerning(3, 3, 12),
      FontGlyphBasicKerning(0, 3, 12),
      FontGlyphBasicKerning(-1, 3, 12),
      FontGlyphBasicKerning(0, 3, 12),
      FontGlyphBasicKerning(0, 3, 12),
      FontGlyphBasicKerning(1, 3, 12),
      FontGlyphBasicKerning(2, 3, 12),
      FontGlyphBasicKerning(1, 3, 12),
      FontGlyphBasicKerning(2, 3, 12),
      FontGlyphBasicKerning(1, 8, 7),
      FontGlyphBasicKerning(1, 8, 7),
      FontGlyphBasicKerning(3, 5, 14),
      FontGlyphBasicKerning(2, 8, 14),
      FontGlyphBasicKerning(3, 5, 14),
      FontGlyphBasicKerning(2, 1, 10),
      FontGlyphBasicKerning(1, 4, 17),
      FontGlyphBasicKerning(-2, 1, 17),
      FontGlyphBasicKerning(1, 1, 16),
      FontGlyphBasicKerning(-1, 1, 16),
      FontGlyphBasicKerning(1, 1, 18),
      FontGlyphBasicKerning(0, 1, 15),
      FontGlyphBasicKerning(1, 1, 15),
      FontGlyphBasicKerning(0, 1, 15),
      FontGlyphBasicKerning(1, 1, 20),
      FontGlyphBasicKerning(1, 1, 11),
      FontGlyphBasicKerning(-1, 1, 12),
      FontGlyphBasicKerning(1, 1, 18),
      FontGlyphBasicKerning(-2, 1, 15),
      FontGlyphBasicKerning(1, 1, 25),
      FontGlyphBasicKerning(1, 1, 18),
      FontGlyphBasicKerning(0, 1, 14),
      FontGlyphBasicKerning(1, 1, 16),
      FontGlyphBasicKerning(0, 1, 15),
      FontGlyphBasicKerning(1, 1, 18),
      FontGlyphBasicKerning(0, 1, 16),
      FontGlyphBasicKerning(1, 1, 15),
      FontGlyphBasicKerning(1, 1, 19),
      FontGlyphBasicKerning(1, 1, 15),
      FontGlyphBasicKerning(1, 1, 22),
      FontGlyphBasicKerning(1, 1, 15),
      FontGlyphBasicKerning(1, 1, 16),
      FontGlyphBasicKerning(1, 1, 14),
      FontGlyphBasicKerning(1, 1, 8),
      FontGlyphBasicKerning(3, 2, 7),
      FontGlyphBasicKerning(-1, 1, 8),
      FontGlyphBasicKerning(3, 3, 14),
      FontGlyphBasicKerning(0, 18, 11),
      FontGlyphBasicKerning(1, 5, 8),
      FontGlyphBasicKerning(-1, 8, 11),
      FontGlyphBasicKerning(0, 1, 9),
      FontGlyphBasicKerning(-1, 8, 8),
      FontGlyphBasicKerning(-1, 3, 11),
      FontGlyphBasicKerning(-1, 8, 8),
      FontGlyphBasicKerning(-2, 3, 7),
      FontGlyphBasicKerning(-1, 8, 10),
      FontGlyphBasicKerning(-1, 3, 10),
      FontGlyphBasicKerning(-1, 6, 6),
      FontGlyphBasicKerning(-2, 4, 8),
      FontGlyphBasicKerning(-1, 3, 11),
      FontGlyphBasicKerning(-1, 3, 7),
      FontGlyphBasicKerning(-1, 8, 16),
      FontGlyphBasicKerning(-1, 8, 10),
      FontGlyphBasicKerning(-1, 8, 9),
      FontGlyphBasicKerning(-3, 6, 11),
      FontGlyphBasicKerning(-1, 8, 10),
      FontGlyphBasicKerning(-2, 8, 8),
      FontGlyphBasicKerning(-2, 7, 8),
      FontGlyphBasicKerning(-2, 6, 6),
      FontGlyphBasicKerning(-1, 8, 11),
      FontGlyphBasicKerning(-1, 8, 9),
      FontGlyphBasicKerning(-1, 8, 14),
      FontGlyphBasicKerning(-1, 8, 8),
      FontGlyphBasicKerning(-1, 8, 10),
      FontGlyphBasicKerning(-2, 8, 8),
      FontGlyphBasicKerning(1, 1, 8),
      FontGlyphBasicKerning(5, 0, 9),
      FontGlyphBasicKerning(-1, 1, 8),
      FontGlyphBasicKerning(2, 10, 13),
      FontGlyphBasicKerning(0, 0, 0),
    };
  }


  UTF8String SmallFont::GetName() const
  {
    return UTF8String("SmallFont");
  }


  UTF8String SmallFont::GetPathName() const
  {
    return UTF8String("FontChars_SmallFont");
  }


  FontDesc SmallFont::GetDesc() const
  {
    return FontDesc(26, 17, Point2(3, 1));
  }


  int32_t SmallFont::RangeCount() const
  {
    return sizeof(g_ranges)/sizeof(FontGlyphRange);
  }


  FontGlyphRange SmallFont::GetRange(const int32_t index) const
  {
    assert(index >= 0 && index <= RangeCount());
    return g_ranges[index];
  }


  int32_t SmallFont::Count() const
  {
    return sizeof(g_glyphKerning)/sizeof(FontGlyphBasicKerning);
  }


  FontGlyphBasicKerning SmallFont::Get(const int32_t index) const
  {
    assert(index >= 0 && index <= Count());
    return g_glyphKerning[index];
  }
}
