/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <iostream>
#include <FslBase/Getopt/OptionBaseValues.hpp>
#include <FslBase/String/StringParseUtil.hpp>
#include <FslDemoHost/ADemoHostOptionParser.hpp>

namespace Fsl
{
  namespace
  {
    struct CommandId
    {
      enum Enum
      {
        Window = DEMO_HOST_OPTION_BASE_INTERNAL,
        DisplayId
      };
    };
  }


  ADemoHostOptionParser::ADemoHostOptionParser()
    : m_nativeWindowConfig(WindowMode::Fullscreen, Rectangle())
    , m_nativeWindowTag()
  {
  }


  void ADemoHostOptionParser::ArgumentSetup(std::deque<Option>& rOptions)
  {
    rOptions.push_back(Option("Window", OptionArgument::OptionRequired, CommandId::Window, "Window mode [left,top,width,height]", OptionGroup::Host));
    rOptions.push_back(Option("DisplayId", OptionArgument::OptionRequired, CommandId::DisplayId, "DisplayId <number>", OptionGroup::Host));
  }


  OptionParseResult::Enum ADemoHostOptionParser::Parse(const int cmdId, const char*const pszOptArg)
  {
    Rectangle rectValue;
    int32_t intValue;
    switch (cmdId)
    {
    case CommandId::Window:
      StringParseUtil::Parse(rectValue, pszOptArg);
      m_nativeWindowConfig.SetWindowMode(WindowMode::Window);
      m_nativeWindowConfig.SetWindowRectangle(rectValue);
      return OptionParseResult::Parsed;
    case CommandId::DisplayId:
      StringParseUtil::Parse(intValue, pszOptArg);
      m_nativeWindowConfig.SetDisplayId(intValue);
      return OptionParseResult::Parsed;
    default:
      return OptionParseResult::NotHandled;
    }
  }


  bool ADemoHostOptionParser::ParsingComplete()
  {
    return true;
  }


  NativeWindowConfig ADemoHostOptionParser::GetNativeWindowConfig() const
  {
    return m_nativeWindowConfig;
  }


  void ADemoHostOptionParser::SetNativeWindowTag(const std::shared_ptr<ITag>& tag)
  {
    m_nativeWindowTag = tag;
  }


  std::shared_ptr<ITag> ADemoHostOptionParser::GetNativeWindowTag() const
  {
    return m_nativeWindowTag;
  }

}


