#ifndef FSLGRAPHICS_PIXELFORMAT_HPP
#define FSLGRAPHICS_PIXELFORMAT_HPP
/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <FslBase/BasicTypes.hpp>

namespace Fsl
{
  struct PixelFormat
  {
    // Internal encoding:
    //   bit 0..15 == format id
    //   bit 16-20 == bytes per pixel
    // However utilize PixelFormatUtil to extract that information as it is subject to change!!!
    enum Enum
    {
      // 24bpp (3bytes) per pixel
      // Memory layout -> rr gg bb
      RGB888 = 0x30000,
      // 24bpp (3bytes) per pixel
      // Memory layout -> bb gg rr
      BGR888 = 0x30001,
      // 32bpp (4bytes) per pixel
      // Memory layout -> rr gg bb aa
      RGBA8888 = 0x40002,
      // 32bpp (4bytes) per pixel
      // Memory layout -> bb gg rr aa
      BGRA8888 = 0x40003,
      // 8bpp (1 byte) per pixel
      // Memory layout -> aa
      ALPHA8 = 0x10004,
      // 8bpp (1 byte) per pixel
      // Memory layout -> ll
      LUMINANCE8 = 0x10005,
      // 16bpp (2 byte) per pixel
      // Memory layout -> ll aa
      LUMINANCE_ALPHA88 = 0x20006
    };
  };
}

#endif


