#if ! defined(__ANDROID__) && defined(__linux__) && ! defined(EGL_USE_X11) && !defined(EGL_API_FB) && ! defined(EGL_API_WL) && defined(EGL_API_DFB) // FIX: use another define
/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

#include <iostream>
#include "EGLNativeWindowDFB.hpp"
#include <FslBase/Log/Log.hpp>
#include <FslGraphicsEGL/Exceptions.hpp>
#include <FslGraphicsEGL/EGLCheck.hpp>
#include <directfb/directfb.h>
#include <pthread.h>
#include <FslGraphics/Window/INativeWindowEventQueue.hpp>
#include <FslGraphics/Window/NativeWindowEventHelper.hpp>

namespace Fsl
{
  namespace
  {
      IDirectFBEventBuffer * globalEventBuffer;
      std::weak_ptr<INativeWindowEventQueue> g_eventQueue;
      struct FSL_DFBDisplay
      {
        IDirectFB*             pDirectFB;
        IDirectFBDisplayLayer* pLayer;
        IDirectFBEventBuffer*  pEventBuffer;
        int  winWidth;
        int  winHeight;
        pthread_cond_t cond;
        pthread_mutex_t condMutex;
      };

      VirtualKey::Enum KeyToVirtualKey(DFBInputDeviceKeyIdentifier keyId)
      {
        switch(keyId)
        {
          case DIKI_A:
            return VirtualKey::A;

          case DIKI_B:
            return VirtualKey::B;

          case DIKI_C:
            return VirtualKey::C;

          case DIKI_D:
            return VirtualKey::D;

          case DIKI_E:
            return VirtualKey::E;

          case DIKI_F:
            return VirtualKey::F;

          case DIKI_G:
            return VirtualKey::G;

          case DIKI_H:
            return VirtualKey::H;

          case DIKI_I:
            return VirtualKey::I;

          case DIKI_J:
            return VirtualKey::J;

          case DIKI_K:
            return VirtualKey::K;

          case DIKI_L:
            return VirtualKey::L;

          case DIKI_M:
            return VirtualKey::M;

          case DIKI_N:
            return VirtualKey::N;

          case DIKI_O:
            return VirtualKey::O;

          case DIKI_P:
            return VirtualKey::P;

          case DIKI_Q:
            return VirtualKey::Q;

          case DIKI_R:
            return VirtualKey::R;

          case DIKI_S:
            return VirtualKey::S;

          case DIKI_T:
            return VirtualKey::T;

          case DIKI_U:
            return VirtualKey::U;

          case DIKI_V:
            return VirtualKey::V;

          case DIKI_W:
            return VirtualKey::W;

          case DIKI_X:
            return VirtualKey::X;

          case DIKI_Y:
            return VirtualKey::Y;

          case DIKI_Z:
            return VirtualKey::Z;

          case DIKI_0:
            return VirtualKey::Code0;

          case DIKI_1:
            return VirtualKey::Code1;

          case DIKI_2:
            return VirtualKey::Code2;

          case DIKI_3:
            return VirtualKey::Code3;

          case DIKI_4:
            return VirtualKey::Code4;

          case DIKI_5:
            return VirtualKey::Code5;

          case DIKI_6:
            return VirtualKey::Code6;

          case DIKI_7:
            return VirtualKey::Code7;

          case DIKI_8:
            return VirtualKey::Code8;

          case DIKI_9:
            return VirtualKey::Code9;

          case DIKI_ESCAPE:
            return VirtualKey::Escape;

          case DIKI_TAB:
            return VirtualKey::Tab;

          case DIKI_ENTER:
            return VirtualKey::Return;

          case DIKI_SPACE:
            return VirtualKey::Space;

          case DIKI_BACKSPACE:
            return VirtualKey::Backspace;

          case DIKI_UP:
            return VirtualKey::UpArrow;

          case DIKI_DOWN:
            return VirtualKey::DownArrow;

          case DIKI_LEFT:
            return VirtualKey::LeftArrow;

          case DIKI_RIGHT:
            return VirtualKey::RightArrow;

          case DIKI_F1:
            return VirtualKey::F1;

          case DIKI_F2:
            return VirtualKey::F2;

          case DIKI_F3:
            return VirtualKey::F3;

          case DIKI_F4:
            return VirtualKey::F4;

          case DIKI_F5:
            return VirtualKey::F5;

          case DIKI_F6:
            return VirtualKey::F6;

          case DIKI_F7:
            return VirtualKey::F7;

          case DIKI_F8:
            return VirtualKey::F8;

          case DIKI_F9:
            return VirtualKey::F9;

          case DIKI_F10:
            return VirtualKey::F10;

          case DIKI_F11:
            return VirtualKey::F11;

          case DIKI_F12:
            return VirtualKey::F12;

          case DIKI_INSERT:
            return VirtualKey::Insert;

          case DIKI_DELETE:
            return VirtualKey::Delete;

          case DIKI_HOME:
            return VirtualKey::Home;

          case DIKI_END:
            return VirtualKey::End;

          case DIKI_PAGE_UP:
            return VirtualKey::PageUp;

          case DIKI_PAGE_DOWN:
            return VirtualKey::PageDown;

          case DIKI_PRINT:
            return VirtualKey::PrintScreen;

          case DIKI_PAUSE:
            return VirtualKey::ScrollLock;

          case DIKI_SCROLL_LOCK:
            return VirtualKey::ScrollLock;

          default :
            return VirtualKey::Unknown;
        }
      }
  }

  EGLNativeWindowSystemDFB::EGLNativeWindowSystemDFB(const NativeWindowSystemSetup& setup)
  {
    FSL_DFBDisplay * local_display;
    FSLLOG_WARNING_IF(setup.GetConfig().GetDisplayId() != 0, "directFB only supports the main display. Using DisplayId 0 instead of " << setup.GetConfig().GetDisplayId());
  
    m_hDisplay = dfbGetDisplay(nullptr);
    if (!m_hDisplay)
      throw EGLGraphicsException("Could not open display");
    local_display = (struct FSL_DFBDisplay *)m_hDisplay;
    //Create a new input buffer
    if(local_display->pDirectFB->CreateInputEventBuffer(local_display->pDirectFB, DICAPS_ALL, DFB_TRUE, &(globalEventBuffer)) != DFB_OK)
    {
      FSLLOG("WARNING: Could not create additional Buffer\n");
    }
  }


  EGLNativeWindowSystemDFB::~EGLNativeWindowSystemDFB()
  {
     dfbDestroyDisplay(m_hDisplay);

  }


  EGLNativeDisplayType EGLNativeWindowSystemDFB::GetDisplayType() const
  {
    return m_hDisplay;
  }


  std::shared_ptr<IEGLNativeWindow> EGLNativeWindowSystemDFB::CreateNativeWindow(const NativeWindowSetup& nativeWindowSetup, const NativeEGLSetup& nativeEglSetup)
  {
    return std::shared_ptr<IEGLNativeWindow>(new EGLNativeWindowDFB(nativeWindowSetup, nativeEglSetup));
  }


  bool EGLNativeWindowSystemDFB::ProcessMessages(const NativeWindowProcessMessagesArgs& args)
  {
    bool bContinue = true;
    static Point2 mousePosition(0,0);
    DFBInputEvent evt;
    VirtualKey::Enum keyCode;
    std::shared_ptr<INativeWindowEventQueue> eventQueue = g_eventQueue.lock();
    while(globalEventBuffer->GetEvent( globalEventBuffer, DFB_EVENT(&evt) ) == DFB_OK)
    {
      //First Handle the event class, as we are using a Global listener
      if(DFEC_INPUT == evt.clazz)
      {
        if (evt.type == DIET_KEYPRESS)
        {
          keyCode = KeyToVirtualKey(evt.key_id);
          if(eventQueue)
            eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputKeyEvent(keyCode, true));
        }
        else if (DIET_KEYRELEASE == evt.type )
        {
          keyCode = KeyToVirtualKey(evt.key_id);
          if(eventQueue)
            eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputKeyEvent(keyCode, false));
        }
        else if (DIET_AXISMOTION == evt.type)
        {
          if (evt.flags & DIEF_AXISREL)
           {
             switch (evt.axis)
             {
               case DIAI_X:
                mousePosition.X += evt.axisrel;
                if(mousePosition.X < 0)
                  mousePosition.X = 0;
               break;
               case DIAI_Y:
                mousePosition.Y += evt.axisrel;
                if(mousePosition.Y < 0)
                  mousePosition.Y = 0;
               break;

               default:
               break;
            }

            if(eventQueue)
              eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseMoveEvent(mousePosition));
          }
        }
        else if (DIET_BUTTONPRESS == evt.type)
        {
          switch (evt.button)
          {
            case DIBI_LEFT:
            {
              if(eventQueue)
                eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseButtonEvent(VirtualMouseButton::Left, true, mousePosition));
            }
            break;

            case DIBI_RIGHT:
            {
              if(eventQueue)
                eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseButtonEvent(VirtualMouseButton::Right, true, mousePosition));
            }
            break;

            case DIBI_MIDDLE:
            {
              if(eventQueue)
                eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseButtonEvent(VirtualMouseButton::Middle, true, mousePosition));
            }
            break;
            default:
            break;
          }
        }
        else if (DIET_BUTTONRELEASE == evt.type)
        {
          switch (evt.button)
          {
            case DIBI_LEFT:
            {
              if(eventQueue)
                eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseButtonEvent(VirtualMouseButton::Left, false, mousePosition));
            }
            break;

            case DIBI_RIGHT:
            {
              if(eventQueue)
                eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseButtonEvent(VirtualMouseButton::Right, false, mousePosition));
            }
            break;

            case DIBI_MIDDLE:
            {
              if(eventQueue)
                eventQueue->PostEvent(NativeWindowEventHelper::EncodeInputMouseButtonEvent(VirtualMouseButton::Middle, false, mousePosition));
            }
            break;

            default:
            break;
          }
        }
      }
    }
    return bContinue;
  }


  EGLNativeWindowDFB::EGLNativeWindowDFB(const NativeWindowSetup& nativeWindowSetup, const NativeEGLSetup& nativeEglSetup)
    : m_hDisplayType(nativeEglSetup.DisplayType)

  {
    const NativeWindowConfig nativeWindowConfig = nativeWindowSetup.GetConfig();
    const EGLDisplay hDisplay = nativeEglSetup.Display;
    const EGLConfig eglConfig = nativeEglSetup.Config;
    int windowWidth, windowHeight, windowX, windowY;
    g_eventQueue = nativeWindowSetup.GetEventQueue();
    FSL_DFBDisplay * local_display;

    // Get the native visual id that matches the egl config
    EGLint nativeVisualId;
    EGL_CHECK(eglGetConfigAttrib(hDisplay, eglConfig, EGL_NATIVE_VISUAL_ID, &nativeVisualId));

    if (nativeWindowConfig.GetWindowMode() != WindowMode::Window)
    {
      FSLLOG("WARNING: Window Size/Position not defined, setting them to MAX Display Resolution");
      local_display = (struct FSL_DFBDisplay *)m_hDisplayType;
      IDirectFBScreen * localScreen;
      if((local_display->pDirectFB->GetScreen(local_display->pDirectFB,0,&(localScreen)))==DFB_OK)
      {
          if((localScreen->GetSize(localScreen,&windowWidth,&windowHeight))!=DFB_OK)
          {
            FSLLOG("WARNING: Display Dimensions could not be retrieved, using predefined values");
            windowWidth = 800;
            windowHeight = 400;
          }
          windowX = 0;
          windowY = 0;
      }
      else
      {
        FSLLOG("WARNING: Display Dimensions could not be retrieved, using predefined values");
        windowWidth = 800;
        windowHeight = 400;
        windowX = 0;
        windowY = 0;
      }
    }
    else
    {
      const Rectangle windowRectangle = nativeWindowConfig.GetWindowRectangle();
      windowWidth  = windowRectangle.Width;
      windowHeight = windowRectangle.Height;
      windowX = windowRectangle.X;
      windowY = windowRectangle.Y;
    }


    m_hWindow = dfbCreateWindow(m_hDisplayType, windowX, windowY, windowWidth, windowHeight);

    if(!m_hWindow)
    {
        throw EGLGraphicsException("Could not create DirectFB Window");
    }

    { // Post the activation message to let the framework know we are ready
      std::shared_ptr<INativeWindowEventQueue> eventQueue = g_eventQueue.lock();
      if( eventQueue )
        eventQueue->PostEvent(NativeWindowEventHelper::EncodeWindowActivationEvent(true));
    }
  }


  EGLNativeWindowDFB::~EGLNativeWindowDFB()
  {
    dfbDestroyWindow(m_hWindow);
  }


  EGLNativeWindowType EGLNativeWindowDFB::GetWindowType() const
  {
    return m_hWindow;
  }

}
#endif
