#ifndef FSLGRAPHICSGLES2_GLTEXTURE_HPP
#define FSLGRAPHICSGLES2_GLTEXTURE_HPP
/****************************************************************************************************************************************************
* Copyright (c) 2014 Freescale Semiconductor, Inc.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*    * Redistributions of source code must retain the above copyright notice,
*      this list of conditions and the following disclaimer.
*
*    * Redistributions in binary form must reproduce the above copyright notice,
*      this list of conditions and the following disclaimer in the documentation
*      and/or other materials provided with the distribution.
*
*    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
*      its contributors may be used to endorse or promote products derived from
*      this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************************************************************************************/

// Make sure Common.hpp is the first include file (to make the error message as helpful as possible when disabled)
#include <FslGraphicsGLES2/Common.hpp>
#include <FslGraphicsGLES2/GLValues.hpp>
#include <GLES2/gl2.h>
#include <FslBase/Noncopyable.hpp>
#include <FslBase/Math/Point2.hpp>
#include <FslGraphics/NativeTextureArea.hpp>
#include <FslGraphics/TextureRectangle.hpp>
#include <FslGraphics/RawBitmap.hpp>
#include <FslGraphics/RawCubeBitmap.hpp>
#include <FslGraphicsGLES2/GLTextureParameters.hpp>
#include <FslGraphicsGLES2/GLTextureInfo.hpp>

namespace Fsl
{
  class Bitmap;

  namespace GLES2
  {

    class GLTexture : private Noncopyable
    {
      GLuint m_handle;
      GLenum m_target;
      Point2 m_size;
    public:

      //! @brief Create a uninitialized texture (use SetData to add texture data to it)
      GLTexture();

      //! @brief Let this GLTexture object assume control over the given texture handle.
      //! @param handle the GL handle to the texture (it's assumed the handle is a GL_TEXTURE_2D)
      //! @param size the size of the texture
      GLTexture(const GLuint handle, const Point2& size);

      //! @brief Let this GLTexture object assume control over the given texture handle.
      //! @param handle the GL handle to the texture
      //! @param the target type specified to glBindTexture when the handle was created.
      //! @param size the size of the texture
      GLTexture(const GLuint handle, const GLenum target, const Point2& size);

      //! @brief Create a texture with the supplied content
      GLTexture(const Bitmap& bitmap, const GLTextureParameters& textureParameters);

      //! @brief Create a texture with the supplied content
      GLTexture(const RawBitmap& bitmap, const GLTextureParameters& textureParameters);

      //! @brief Create a texture with the supplied content
      GLTexture(const Bitmap& bitmap, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Create a texture with the supplied content
      GLTexture(const RawBitmap& bitmap, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Create a texture with the supplied content
      GLTexture(const Bitmap& bitmapPosX, const Bitmap& bitmapNegX, const Bitmap& bitmapPosY, const Bitmap& bitmapNegY, const Bitmap& bitmapPosZ, const Bitmap& bitmapNegZ, const GLTextureParameters& textureParameters);

      //! @brief Create a texture with the supplied content
      GLTexture(const RawBitmap& bitmapPosX, const RawBitmap& bitmapNegX, const RawBitmap& bitmapPosY, const RawBitmap& bitmapNegY, const RawBitmap& bitmapPosZ, const RawBitmap& bitmapNegZ, const GLTextureParameters& textureParameters);

      //! @brief Create a texture with the supplied content
      GLTexture(const RawCubeBitmap& cubeBitmap, const GLTextureParameters& textureParameters);

      //! @brief Create a texture with the supplied content
      GLTexture(const Bitmap& bitmapPosX, const Bitmap& bitmapNegX, const Bitmap& bitmapPosY, const Bitmap& bitmapNegY, const Bitmap& bitmapPosZ, const Bitmap& bitmapNegZ, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Create a texture with the supplied content
      GLTexture(const RawBitmap& bitmapPosX, const RawBitmap& bitmapNegX, const RawBitmap& bitmapPosY, const RawBitmap& bitmapNegY, const RawBitmap& bitmapPosZ, const RawBitmap& bitmapNegZ, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Create a texture with the supplied content
      GLTexture(const RawCubeBitmap& cubeBitmap, const GLTextureParameters& textureParameters, const bool generateMipMaps);



      ~GLTexture();

      //! @brief Check if this contains a valid gl handle.
      bool IsValid() const { return m_handle != GLValues::INVALID_HANDLE; }


      //! @brief If a texture is allocated this will releases it.
      void Reset();

      //! @brief Let this GLTexture object assume control over the given texture handle.
      //! @param handle the GL handle to the texture (it's assumed the handle is a GL_TEXTURE_2D)
      //! @param size the size of the texture
      void Reset(const GLuint handle, const Point2& size);

      //! @brief Let this GLTexture object assume control over the given texture handle.
      //! @param handle the GL handle to the texture
      //! @param the target type specified to glBindTexture when the handle was created.
      //! @param size the size of the texture
      void Reset(const GLuint handle, const GLenum target, const Point2& size);

      //! @brief Reset the data of the texture
      void Reset(const Bitmap& bitmap, const GLTextureParameters& textureParameters);

      //! @brief Reset the data of the texture
      void Reset(const RawBitmap& bitmap, const GLTextureParameters& textureParameters);

      //! @brief Reset the data of the texture
      void Reset(const Bitmap& bitmap, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Reset the data of the texture
      void Reset(const RawBitmap& bitmap, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Reset the data of the texture
      void Reset(const Bitmap& bitmapPosX, const Bitmap& bitmapNegX, const Bitmap& bitmapPosY, const Bitmap& bitmapNegY, const Bitmap& bitmapPosZ, const Bitmap& bitmapNegZ, const GLTextureParameters& textureParameters);

      //! @brief Reset the data of the texture
      void Reset(const RawBitmap& bitmapPosX, const RawBitmap& bitmapNegX, const RawBitmap& bitmapPosY, const RawBitmap& bitmapNegY, const RawBitmap& bitmapPosZ, const RawBitmap& bitmapNegZ, const GLTextureParameters& textureParameters);

      //! @brief Reset the data of the texture
      void Reset(const RawCubeBitmap& cubeBitmap, const GLTextureParameters& textureParameters);

      //! @brief Reset the data of the texture
      void Reset(const Bitmap& bitmapPosX, const Bitmap& bitmapNegX, const Bitmap& bitmapPosY, const Bitmap& bitmapNegY, const Bitmap& bitmapPosZ, const Bitmap& bitmapNegZ, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Reset the data of the texture
      void Reset(const RawBitmap& bitmapPosX, const RawBitmap& bitmapNegX, const RawBitmap& bitmapPosY, const RawBitmap& bitmapNegY, const RawBitmap& bitmapPosZ, const RawBitmap& bitmapNegZ, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Reset the data of the texture
      void Reset(const RawCubeBitmap& cubeBitmap, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Set the data of the texture
      void SetData(const Bitmap& bitmap, const GLTextureParameters& textureParameters);

      //! @brief Set the data of the texture
      void SetData(const RawBitmap& bitmap, const GLTextureParameters& textureParameters);

      //! @brief Set the data of the texture
      void SetData(const Bitmap& bitmap, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Set the data of the texture
      void SetData(const RawBitmap& bitmap, const GLTextureParameters& textureParameters, const bool generateMipMaps);
      
      //! @brief Set the data of the texture
      void SetData(const Bitmap& bitmapPosX, const Bitmap& bitmapNegX, const Bitmap& bitmapPosY, const Bitmap& bitmapNegY, const Bitmap& bitmapPosZ, const Bitmap& bitmapNegZ, const GLTextureParameters& textureParameters);

      //! @brief Set the data of the texture
      void SetData(const RawBitmap& bitmapPosX, const RawBitmap& bitmapNegX, const RawBitmap& bitmapPosY, const RawBitmap& bitmapNegY, const RawBitmap& bitmapPosZ, const RawBitmap& bitmapNegZ, const GLTextureParameters& textureParameters);

      //! @brief Set the data of the texture
      void SetData(const RawCubeBitmap& cubeBitmap, const GLTextureParameters& textureParameters);

      //! @brief Set the data of the texture
      void SetData(const Bitmap& bitmapPosX, const Bitmap& bitmapNegX, const Bitmap& bitmapPosY, const Bitmap& bitmapNegY, const Bitmap& bitmapPosZ, const Bitmap& bitmapNegZ, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Set the data of the texture
      void SetData(const RawBitmap& bitmapPosX, const RawBitmap& bitmapNegX, const RawBitmap& bitmapPosY, const RawBitmap& bitmapNegY, const RawBitmap& bitmapPosZ, const RawBitmap& bitmapNegZ, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Set the data of the texture
      void SetData(const RawCubeBitmap& cubeBitmap, const GLTextureParameters& textureParameters, const bool generateMipMaps);

      //! @brief Get the gl texture handle associated with the texture.
      //! @return the handle or GLValues::INVALID_HANDLE if the texture is unallocated.
      GLuint GetHandle() const;

      //! @brief Get the gl texture target associated with the texture.
      //! @return the handle or GLValues::INVALID_HANDLE if the texture is unallocated.
      GLuint GetTarget() const;

      //! @brief Get the texture size.
      Point2 GetSize() const;

      //! @brief Extract information about this texture as a GLTextureInfo struct
      GLTextureInfo GetTextureInfo() const { return GLTextureInfo(m_handle, m_size); }

      //! @brief Extract information about this texture as a GLTextureInfo struct
      operator GLTextureInfo() { return GLTextureInfo(m_handle, m_size); }

      //! @brief Get the native texture area of the given textureRectangle
      static NativeTextureArea CalcTextureArea(const TextureRectangle& textureRectangle, const int timesX, const int timesY);
    };

  }
}

#endif 
